/*
 * mesg.h
 *
 * Header file for the mesg library.
 *
 * Kekoa Proudfoot 11/29/96
 */

#ifndef MESG_H_DEFINED
#define MESG_H_DEFINED

#include <stdio.h>

/* These should be in a qtypes.h file, along with vec_t and rot_t */

#define MESG_NUM_MODEL_NAMES  256
#define MESG_NUM_SOUND_NAMES  256

#define MESG_BLOCK_MAX_SIZE   16384

typedef struct vec_t {
    float x;
    float y;
    float z;
} vec_t;

typedef struct rot_t {
    float p;
    float y;
    float r;
} rot_t;

typedef enum stat_index_t {
    STAT_HEALTH          = 0,
    STAT_WEAPON_MODEL    = 2,
    STAT_CURRENT_AMMO    = 3,
    STAT_ARMOR           = 4,
    STAT_WEAPON_FRAME    = 5,
    STAT_AMMO_SHELLS     = 6,
    STAT_AMMO_NAILS      = 7,
    STAT_AMMO_ROCKETS    = 8,
    STAT_AMMO_CELLS      = 9,
    STAT_CURRENT_WEAPON  = 10,
    STAT_TOTAL_SECRETS   = 11,
    STAT_TOTAL_MONSTERS  = 12,
    STAT_FOUND_SECRETS   = 13,
    STAT_FOUND_MONSTERS  = 14
} stat_index_t;

typedef enum item_flag_t {
    ITEM_SHOTGUN            = 0x00000001,
    ITEM_SUPER_SHOTGUN      = 0x00000002,
    ITEM_NAILGUN            = 0x00000004,
    ITEM_SUPER_NAILGUN      = 0x00000008,
    ITEM_GRENADE_LAUNCHER   = 0x00000010,
    ITEM_ROCKET_LAUNCHER    = 0x00000020,
    ITEM_LIGHTNING          = 0x00000040,
    ITEM_EXTRA_WEAPON       = 0x00000080,
    ITEM_SHELLS_ACTIVE      = 0x00000100,
    ITEM_NAILS_ACTIVE       = 0x00000200,
    ITEM_ROCKETS_ACTIVE     = 0x00000400,
    ITEM_CELLS_ACTIVE       = 0x00000800,
    ITEM_AXE                = 0x00001000,
    ITEM_ARMOR_1            = 0x00002000,
    ITEM_ARMOR_2            = 0x00004000,
    ITEM_ARMOR_3            = 0x00008000,
    ITEM_SUPER_HEALTH       = 0x00010000,
    ITEM_KEY_1              = 0x00020000,
    ITEM_KEY_2              = 0x00040000,
    ITEM_INVISIBILITY       = 0x00080000,
    ITEM_INVULNERABILITY    = 0x00100000,
    ITEM_SUIT               = 0x00200000,
    ITEM_QUAD               = 0x00400000,
    ITEM_RUNE_1             = 0x10000000,
    ITEM_RUNE_2             = 0x20000000,
    ITEM_RUNE_3             = 0x40000000,
    ITEM_RUNE_4             = 0x80000000
} item_flag_t;

typedef enum weapon_flag_t {
    WEAPON_AXE               = 0x00,
    WEAPON_SHOTGUN           = 0x01,
    WEAPON_SUPER_SHOTGUN     = 0x02,
    WEAPON_NAILGUN           = 0x04,
    WEAPON_SUPER_NAILGUN     = 0x08,
    WEAPON_GRENADE_LAUNCHER  = 0x10,
    WEAPON_ROCKET_LAUNCHER   = 0x20,
    WEAPON_LIGHTNING         = 0x40,
    WEAPON_EXTRA             = 0x80
} weapon_flag_t;

/*****************************************************************************
 * Type definitions
 *****************************************************************************/

/* Enum of message types */

typedef enum mesg_type_t {

    /* Shared message types */

    MESG_BAD                 = 0x00,

    /* Client to server message types */

    MESG_CLIENT_CONNECT      = 0x01,
    MESG_CLIENT_DISCONNECT   = 0x02,
    MESG_SERVER_DATA         = 0x03,
    MESG_EXECUTE_COMMAND     = 0x04,

    /* Server to client message types */

    MESG_NOP                 = 0x01,
    MESG_DISCONNECT          = 0x02,
    MESG_UPDATE_STAT         = 0x03,
    MESG_VERSION             = 0x04,
    MESG_SET_VIEW            = 0x05,
    MESG_SOUND               = 0x06,
    MESG_TIME                = 0x07,
    MESG_PRINT               = 0x08,
    MESG_STUFF_TEXT          = 0x09,
    MESG_SET_ANGLE           = 0x0a,
    MESG_SERVER_INFO         = 0x0b,
    MESG_LIGHT_STYLE         = 0x0c,
    MESG_UPDATE_NAME         = 0x0d,
    MESG_UPDATE_FRAGS        = 0x0e,
    MESG_CLIENT_DATA         = 0x0f,
    MESG_STOP_SOUND          = 0x10,
    MESG_UPDATE_COLORS       = 0x11,
    MESG_PARTICLE            = 0x12,
    MESG_DAMAGE              = 0x13,
    MESG_SPAWN_STATIC        = 0x14,
    MESG_SPAWN_BINARY        = 0x15,
    MESG_SPAWN_BASELINE      = 0x16,
    MESG_TEMP_ENTITY         = 0x17,
    MESG_SET_PAUSE           = 0x18,
    MESG_SIGNON_NUM          = 0x19,
    MESG_CENTER_PRINT        = 0x1a,
    MESG_KILLED_MONSTER      = 0x1b,
    MESG_FOUND_SECRET        = 0x1c,
    MESG_SPAWN_STATIC_SOUND  = 0x1d,
    MESG_INTERMISSION        = 0x1e,
    MESG_FINALE              = 0x1f,
    MESG_CD_TRACK            = 0x20,
    MESG_SELL_SCREEN         = 0x21,
    MESG_UPDATE_ENTITY       = 0x80
} mesg_type_t;

/* Shared message structs */

typedef struct mesg_bad_t {
    char  type;
} mesg_bad_t;

/* Client to server message structs */

typedef struct mesg_client_connect_t {
    char type;
} mesg_client_connect_t;

typedef struct mesg_client_disconnect_t {
    char type;
} mesg_client_disconnect_t;

typedef enum mesg_server_data_flag_t {
    MESG_SERVER_DATA_FLAG_FIRE  = 0x01,
    MESG_SERVER_DATA_FLAG_JUMP  = 0x02
} mesg_server_data_flag_t;

typedef struct mesg_server_data_t {
    char  type;
    float time;
    rot_t orientation;
    vec_t velocity;
    char  flags;
    char  impulse;
} mesg_server_data_t;

typedef struct mesg_execute_command_t {
    char  type;
    char *text;
} mesg_execute_command_t;

/* Server to client message structs */

typedef struct mesg_nop_t {
    char  type;
} mesg_nop_t;

typedef struct mesg_disconnect_t {
    char  type;
} mesg_disconnect_t;

typedef struct mesg_update_stat_t {
    char  type;
    char  index;
    long  value;
} mesg_update_stat_t;

typedef struct mesg_version_t {
    char  type;
    long  version;
} mesg_version_t;

typedef struct mesg_set_view_t {
    char  type;
    short entity;
} mesg_set_view_t;

typedef struct mesg_sound_t {
    char  type;
    char  mask;
    float volume;
    char  attenuation;
    char  channel;
    short entity;
    char  sound;
    vec_t position;
} mesg_sound_t;

typedef struct mesg_time_t {
    char  type;
    float time;
} mesg_time_t;

typedef struct mesg_print_t {
    char  type;
    char *text;
} mesg_print_t;

typedef struct mesg_stuff_text_t {
    char  type;
    char *text;
} mesg_stuff_text_t;

typedef struct mesg_set_angle_t {
    char  type;
    rot_t orientation;
} mesg_set_angle_t;

typedef struct mesg_server_info_t {
    char  type;
    long  version;
    char  max_clients;
    char  multi_player;
    char *map_name;
    short num_models;
    char *model_names[MESG_NUM_MODEL_NAMES];
    short num_sounds;
    char *sound_names[MESG_NUM_SOUND_NAMES];
} mesg_server_info_t;

typedef struct mesg_light_style_t {
    char  type;
    char  style;
    char *text;
} mesg_light_style_t;

typedef struct mesg_update_name_t {
    char  type;
    char  player;
    char *name;
} mesg_update_name_t;

typedef struct mesg_update_frags_t {
    char  type;
    char  player;
    short frags;
} mesg_update_frags_t;

typedef struct mesg_client_data_t {
    char  type;
    short mask;
    char  view_z_offset;
    char  view_pitch_offset;
    vec_t velocity;
    rot_t orientation;
    long  items;
    char  bits;
    char  weapon_frame;
    char  weapon_model;
    char  armor;
    short health;
    char  current_ammo;
    char  ammo_shells;
    char  ammo_nails;
    char  ammo_rockets;
    char  ammo_cells;
    char  current_weapon;
} mesg_client_data_t;

typedef struct mesg_stop_sound_t {
    char  type;
    short sound;
} mesg_stop_sound_t;

typedef struct mesg_update_colors_t {
    char  type;
    char  player;
    char  shirt_color;
    char  pants_color;
} mesg_update_colors_t;

typedef struct mesg_particle_t {
    char  type;
    vec_t position;
    vec_t velocity;
    char  color;
    char  count;
} mesg_particle_t;

typedef struct mesg_damage_t {
    char  type;
    char  armor_damage;
    char  health_damage;
    vec_t position;
} mesg_damage_t;

typedef struct mesg_spawn_static_t {
    char  type;
    char  default_model;
    char  default_frame;
    char  default_colormap;
    char  default_skin;
    vec_t default_position;
    rot_t default_orientation;
} mesg_spawn_static_t;

typedef struct mesg_spawn_binary_t {
    char  type;
} mesg_spawn_binary_t;

typedef struct mesg_spawn_baseline_t {
    char  type;
    short entity;
    char  default_model;
    char  default_frame;
    char  default_colormap;
    char  default_skin;
    vec_t default_position;
    rot_t default_orientation;
} mesg_spawn_baseline_t;

typedef enum mesg_temp_entity_type_t {
    MESG_TEMP_ENTITY_SPIKE           = 0,
    MESG_TEMP_ENTITY_SUPER_SPIKE     = 1,
    MESG_TEMP_ENTITY_GUN_SHOT        = 2,
    MESG_TEMP_ENTITY_EXPLOSION       = 3,
    MESG_TEMP_ENTITY_TAR_EXPLOSION   = 4,
    MESG_TEMP_ENTITY_SHAMBLER_BOLT   = 5,
    MESG_TEMP_ENTITY_LIGHTNING_BOLT  = 6,
    MESG_TEMP_ENTITY_WIZ_SPIKE       = 7,
    MESG_TEMP_ENTITY_KNIGHT_SPIKE    = 8,
    MESG_TEMP_ENTITY_CHTHON_BOLT     = 9,
    MESG_TEMP_ENTITY_LAVA_SPLASH     = 10,
    MESG_TEMP_ENTITY_TELEPORTER      = 11
} mesg_temp_entity_type_t;

typedef struct mesg_temp_entity_t {
    char  type;
    short entity;
    char  entity_type;
    vec_t position;
    vec_t destination;
} mesg_temp_entity_t;

typedef struct mesg_set_pause_t {
    char  type;
    char  pause;
} mesg_set_pause_t;

typedef struct mesg_signon_num_t {
    char  type;
    char  signon_num;
} mesg_signon_num_t;

typedef struct mesg_center_print_t {
    char  type;
    char *text;
} mesg_center_print_t;

typedef struct mesg_killed_monster_t {
    char  type;
} mesg_killed_monster_t;

typedef struct mesg_found_secret_t {
    char  type;
} mesg_found_secret_t;

typedef struct mesg_spawn_static_sound_t {
    char  type;
    vec_t position;
    char  sound;
    float volume;
    char  attenuation;
} mesg_spawn_static_sound_t;

typedef struct mesg_intermission_t {
    char  type;
} mesg_intermission_t;

typedef struct mesg_finale_t {
    char  type;
    char *text;
} mesg_finale_t;

typedef struct mesg_cd_track_t {
    char  type;
    char  first_track;
    char  last_track;
} mesg_cd_track_t;

typedef struct mesg_sell_screen_t {
    char  type;
} mesg_sell_screen_t;

typedef enum mesg_update_entity_mask_t {
    MESG_UPDATE_ENTITY_MASK_POSITION_X         = 0x0002,
    MESG_UPDATE_ENTITY_MASK_POSITION_Y         = 0x0004,
    MESG_UPDATE_ENTITY_MASK_POSITION_Z         = 0x0008,
    MESG_UPDATE_ENTITY_MASK_ORIENTATION_YAW    = 0x0010,
    MESG_UPDATE_ENTITY_MASK_NEW                = 0x0020,
    MESG_UPDATE_ENTITY_MASK_FRAME              = 0x0040,
    MESG_UPDATE_ENTITY_MASK_ORIENTATION_PITCH  = 0x0100,
    MESG_UPDATE_ENTITY_MASK_ORIENTATION_ROLL   = 0x0200,
    MESG_UPDATE_ENTITY_MASK_MODEL_INDEX        = 0x0400,
    MESG_UPDATE_ENTITY_MASK_COLORMAP           = 0x0800,
    MESG_UPDATE_ENTITY_MASK_SKIN               = 0x1000,
    MESG_UPDATE_ENTITY_MASK_ATTACK_STATE       = 0x2000
} mesg_update_entity_mask_t;

typedef struct mesg_update_entity_t {
    char  type;
    short mask;
    short entity;
    char  model;
    char  frame;
    char  colormap;
    char  skin;
    char  attack_state;
    vec_t position;
    rot_t orientation;
    char  new;
} mesg_update_entity_t;

/* Union of all client to server message structs */

typedef union cl_to_sv_mesg_t {
    char                       type;
    mesg_bad_t                 bad;
    mesg_client_connect_t      client_connect;
    mesg_client_disconnect_t   client_disconnect;
    mesg_server_data_t         server_data;
    mesg_execute_command_t     execute_command;
} cl_to_sv_mesg_t;

/* Union of all server to client message structs */

typedef union sv_to_cl_mesg_t {
    char                       type;
    mesg_bad_t                 bad;
    mesg_nop_t                 nop;
    mesg_disconnect_t          disconnect;
    mesg_update_stat_t         update_stat;
    mesg_version_t             version;
    mesg_set_view_t            set_view;
    mesg_sound_t               sound;
    mesg_time_t                time;
    mesg_print_t               print;
    mesg_stuff_text_t          stuff_text;
    mesg_set_angle_t           set_angle;
    mesg_server_info_t         server_info;
    mesg_light_style_t         light_style;
    mesg_update_name_t         update_name;
    mesg_update_frags_t        update_frags;
    mesg_client_data_t         client_data;
    mesg_stop_sound_t          stop_sound;
    mesg_update_colors_t       update_colors;
    mesg_particle_t            particle;
    mesg_damage_t              damage;
    mesg_spawn_static_t        spawn_static;
    mesg_spawn_binary_t        spawn_binary;
    mesg_spawn_baseline_t      spawn_baseline;
    mesg_temp_entity_t         temp_entity;
    mesg_set_pause_t           set_pause;
    mesg_signon_num_t          signon_num;
    mesg_center_print_t        center_print;
    mesg_killed_monster_t      killed_monster;
    mesg_found_secret_t        found_secret;
    mesg_spawn_static_sound_t  spawn_static_sound;
    mesg_intermission_t        intermission;
    mesg_finale_t              finale;
    mesg_cd_track_t            cd_track;
    mesg_sell_screen_t         sell_screen;
    mesg_update_entity_t       update_entity;
} sv_to_cl_mesg_t;

/*****************************************************************************
 * Interface function declarations
 *****************************************************************************/

extern int   cl_to_sv_mesg_get  (char *buf, cl_to_sv_mesg_t *mesg);
extern int   cl_to_sv_mesg_put  (char *buf, cl_to_sv_mesg_t *mesg);
extern char *cl_to_sv_mesg_name (cl_to_sv_mesg_t *mesg);
extern void  cl_to_sv_mesg_debug(cl_to_sv_mesg_t *mesg, FILE *file);
extern char  cl_to_sv_mesg_peek (char *buf);
extern int   cl_to_sv_mesg_skip (char *buf);

extern int   sv_to_cl_mesg_get  (char *buf, sv_to_cl_mesg_t *mesg);
extern int   sv_to_cl_mesg_put  (char *buf, sv_to_cl_mesg_t *mesg);
extern char *sv_to_cl_mesg_name (sv_to_cl_mesg_t *mesg);
extern void  sv_to_cl_mesg_debug(sv_to_cl_mesg_t *mesg, FILE *file);
extern char  sv_to_cl_mesg_peek (char *buf);
extern int   sv_to_cl_mesg_skip (char *buf);

#endif
