#include "main.h"

void 
send_connection_request(void)
{
  packet_t       *packet;
  cl_to_sv_ctrl_t ctrl;
 
  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending CONNECTION REQUEST\n");
  }

  packet                       = packet_create(PACKET_CONTROL);
  ctrl.op                      = CTRL_REQ_CONNECT;
  ctrl.req_connect.game        = "QUAKE";
  ctrl.req_connect.net_version = 3;
  packet->header.length        = cl_to_sv_ctrl_put(packet->control.data, &ctrl);
  send_packet(packet);
  packet_destroy(packet);
}

void 
send_acknowledge(void)
{
  packet_t *packet;

/*   if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) { */
/*     print_to_log("Sending ACK\n"); */
/*   } */

  packet                = packet_create(PACKET_RELIABLE_REPLY);
  packet->header.number = gs.recv_reliable_number;

  send_packet(packet);
  packet_destroy(packet);
}

void 
send_disconnect(void) 
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if( !gs.server_socket ) {
     return;
  }

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending DISCONNECT\n");
  }

  packet                     = packet_create(PACKET_UNRELIABLE);

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                  = MESG_CLIENT_DISCONNECT;

  packet->unreliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);
  packet->unreliable.number  = gs.send_unreliable_number++;

  send_packet(packet);
  packet_destroy(packet);

  reset_connection();
}

void 
send_keepalive(void)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending KEEPALIVE\n");
  }

  packet                     = packet_create(PACKET_UNRELIABLE);

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                  = MESG_NOP;

  packet->unreliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

  queue_add(&gs.send_unreliable_queue, packet);
}

void 
send_impulse(int impulse)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if( impulse >= 0 && impulse < 256 ) {
    if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
      print_to_log("Sending IMPULSE %d\n", impulse);
    }

    packet                     = packet_create(PACKET_UNRELIABLE);

    memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
    mesg.type                  = MESG_SERVER_DATA;
    mesg.server_data.impulse   = (unsigned char)impulse;

    packet->unreliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

    queue_add(&gs.send_unreliable_queue, packet);
  }
}

void 
send_fire(void)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending FIRE\n");
  }

  packet                     = packet_create(PACKET_UNRELIABLE);

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                  = MESG_SERVER_DATA;
  mesg.server_data.flags     = MESG_SERVER_DATA_FLAG_FIRE;

  packet->unreliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

  queue_add(&gs.send_unreliable_queue, packet);
}

void 
send_jump(void)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending JUMP\n");
  }

  packet                     = packet_create(PACKET_UNRELIABLE);

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                  = MESG_SERVER_DATA;
  mesg.server_data.flags     = MESG_SERVER_DATA_FLAG_JUMP;

  packet->unreliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

  queue_add(&gs.send_unreliable_queue, packet);
}

void 
send_unreliable_head(void)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  if( is_in_the_game() ) {
    if( !queue_empty(&gs.send_unreliable_queue)) {
      packet = (packet_t *)queue_remove(&gs.send_unreliable_queue);
    } else {
      packet                   = &gs.filler_packet;
    }

    packet->unreliable.number  = gs.send_unreliable_number++;
    cl_to_sv_mesg_get(packet->unreliable.data, &mesg);
    mesg.server_data.time      = gs.time;
    cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

    send_packet(packet);
    if( packet != &gs.filler_packet ) {
      packet_destroy(packet);
    }
  }
}

void
init_filler_packet(void)
{
  cl_to_sv_mesg_t mesg;

  gs.filler_packet.type              = PACKET_UNRELIABLE;
  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                          = MESG_SERVER_DATA;

  gs.filler_packet.unreliable.length = 
           cl_to_sv_mesg_put(gs.filler_packet.unreliable.data, &mesg);
}

void
send_name_command(char *name)
{
  if( strlen(name) == 0 && gs.allow_null_name == FALSE ) {
    send_console_command("name \" \"");
  } else {
    send_console_command("name \"%s\"", name);
  }
}

void 
send_console_command(char *fmt, ...)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;
  va_list         arglist;
  char            command[512];

  packet                     = packet_create(PACKET_RELIABLE_END);
  packet->reliable.number    = gs.send_reliable_number++;

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                  = MESG_EXECUTE_COMMAND;
  va_start(arglist, fmt);
  vsprintf(command, fmt, arglist);
  mesg.execute_command.text  = command;
  va_end(arglist);

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending CONSOLE COMMAND: %s\n", command);
  }

  packet->reliable.length    = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

  send_reliable_packet(packet);
}

void 
send_mesg_client_connect(void)
{
  packet_t       *packet;
  cl_to_sv_mesg_t mesg;

  packet                   = packet_create(PACKET_RELIABLE_END);
  packet->reliable.number  = gs.send_reliable_number++;

  memset((char *)&mesg, 0, sizeof(cl_to_sv_mesg_t));
  mesg.type                = MESG_CLIENT_CONNECT;

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending CLIENT CONNECT\n");
  }

  packet->reliable.length  = cl_to_sv_mesg_put(packet->unreliable.data, &mesg);

  send_reliable_packet(packet);
}

void 
send_mesg_signon_2(void)
{
  packet_t          *packet;
  cl_to_sv_mesg_t    mesg;
  int                len = 0;
  char               tmp[100];

  packet                     = packet_create(PACKET_RELIABLE_END);

  mesg.type                  = MESG_EXECUTE_COMMAND;
  sprintf(tmp,"name \"%s\"\n", gs.player_name);
  mesg.execute_command.text  = tmp;
  len                       += cl_to_sv_mesg_put(&packet->reliable.data[len], &mesg);

  mesg.type                  = MESG_EXECUTE_COMMAND;
  sprintf(tmp,"color %d %d\n", gs.player_shirt, gs.player_pants);
  mesg.execute_command.text  = tmp;
  len                       += cl_to_sv_mesg_put(&packet->reliable.data[len], &mesg);

  mesg.type                  = MESG_EXECUTE_COMMAND;
  mesg.execute_command.text  = "spawn ";
  len                       += cl_to_sv_mesg_put(&packet->reliable.data[len], &mesg);

  if (debug_check_mode(DEBUG_OUTGOING_COMMANDS)) {
    print_to_log("Sending CLIENT INFO: name \"%s\", color %d %d, spawn \n",
       gs.player_name, gs.player_shirt, gs.player_pants);
  }

  packet->reliable.length    = len;
  packet->reliable.number    = gs.send_reliable_number++;
 
  send_reliable_packet(packet);
}

void 
send_reliable_packet(packet_t *packet)
{
  if( queue_empty(&gs.send_reliable_queue) ) {
    queue_add(&gs.send_reliable_queue, packet);
    send_reliable_head();
  } else {
    queue_add(&gs.send_reliable_queue, packet);
  }
}

void 
send_reliable_head(void)
{
  if( !queue_empty(&gs.send_reliable_queue) ) {
    send_packet((packet_t *)queue_get_head(&gs.send_reliable_queue));
  }
}

void 
send_packet(packet_t *packet)
{
  if( is_connected() ) {
    if (debug_check_mode(DEBUG_OUTGOING_PACKET_DATA)) {
      packet_cl_to_sv_debug(packet, gs.logfp);
    }

    if( !net_send(gs.server_socket, packet) ) {
      print_to_log("Send error to quake server: %s\n", strerror(errno));
      send_to_clients("exception Send error to quake server: %s\n", strerror(errno));
      reset_connection();
    }
  }
}
